/**
  GPU Mandelbrot Application
  
  Orion Sky Lawlor, olawlor@acm.org, 2010-01-22 (Public Domain)
*/
#include <iostream>
#include <cmath>
#include <GL/glew.h> /* 
OpenGL Extensions Wrangler:  for gl...ARB extentions.  Must call glewInit after glutCreateWindow! */
#include <GL/glut.h> /* OpenGL Utilities Toolkit, for GUI tools */
#include "ogl/glsl.h"
#include "ogl/glsl.cpp"
#include "ogl/minicam.h"
#include "osl/mat4.h"
#include "osl/mat4_inverse.h"

void display(void) 
{
	glDisable(GL_DEPTH_TEST);
	glDepthMask(GL_TRUE); // turn on Z writes
	glDisable(GL_BLEND);
	
	glClearColor(0.4,0.5,0.7,0.0); // background color
	glClear(GL_COLOR_BUFFER_BIT);// + GL_DEPTH_BUFFER_BIT);

        glViewport(0,0, glutGet(GLUT_WINDOW_WIDTH),glutGet(GLUT_WINDOW_HEIGHT));
	
	float size=50.0f*camera.z;
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity(); // flush any ancient matrices
	gluPerspective(70.0, // fov
		glutGet(GLUT_WINDOW_WIDTH)/(float)glutGet(GLUT_WINDOW_HEIGHT),
		0.000001,
		size); // z clipping planes
	float camera_speed=camera.z;
	oglCameraLookAt(camera_speed);
	if (camera.z<0.0) camera.z=0.000001;
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity(); // flush any ancient matrices
	
	/* Build the pixel shader */
	static GLhandleARB prog=makeProgramObjectFromFiles(
		"shader_vertex.txt","shader.txt");
	glUseProgramObjectARB(prog);
	glFastUniform3fv(prog,"C",1,camera);

	glTranslatef(camera.x,camera.y,0.0); // proxy geom. centered on camera
	glBegin(GL_TRIANGLE_FAN);
	glVertex2f(-size,-size);
	glVertex2f(+size,-size);
	glVertex2f(+size,+size);
	glVertex2f(-size,+size);
	glEnd();
	
	glUseProgramObjectARB(0);
	
	glutPostRedisplay(); // continual animation
	glutSwapBuffers();
}

int main(int argc,char *argv[]) 
{
	glutInit(&argc,argv);
	glutInitDisplayMode(GLUT_RGBA + GLUT_DEPTH + GLUT_DOUBLE);
	glutInitWindowSize(8000,4000);
	glutCreateWindow("GPU Mandelbrot");
	
	glewInit();
	
	glutDisplayFunc(display);
	oglCameraInit();
	camera=vec3(0,0,1);
	camera_orient.z=vec3(0,0,1);
	camera_orient.x=vec3(1,0,0);
	camera_orient.y=vec3(0,1,0);
	
	glutMainLoop();
	return 0;
}
